// main file for  RDS USB  coder ATm32 software 
// =====================================================
//  Description
//    This file include main task of RDS USB software 
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
//
//=======================================
// Author : G.Laroche www.g.laroche.free.fr
// =====================================================
// history
//  19/05/07    V 1.0
//     First version, create 
//  04/08/07    V 1.1
//     translate RS232 help menu in english 
// =====================================================





/* include system */
#include <avr/io.h>
#include <avr/delay.h>
#include <avr/interrupt.h>
#include <stdio.h>
#include <stdlib.h>
/* pgmspace.h is necessary to use char string in flash memory */
/* PSTR("string")  to direct flash using, else String are duplicate into RAM  !!!! */
#include <avr/pgmspace.h>   


/* include application */
#include "RDS.h"
#include "RDS_ClockDriver.h"
#include "RDS_RS232.h"
#include "RDS_IOMgt.h"
#include "RDS_EepromMgt.h"
#include "RDS_GroupsGenerate.h"
#include "RDS_USBInterface.h"



/* global variable */
/* RDSDumpBuf if !=0 dump RDS buffer */
unsigned char RDSDumpBuf;

/* Local Functions */

void InitAppli(void)
{
    RDSDumpBuf=0 ;  
	
	/* initialise RS232 driver */
	RS_Init(38);    /* 12MHz crystal and 19200 Bds  with U2X=0*/
	                /* UBRR= (Fosc/(16*19200)) -1 --> (12000000/307200)-1 = 38 */
    /* initialise EEPROM mgt */
    EEP_Init();
    /* initialise IO mgt */
    RDSIO_Init();
    
    /* initialise RDS clock driver */
	RDSCLK_Init();
	
	RDSGEN_Init();
		
	#ifdef RDS_SOFT_BIPHASE
//    RS_PrintfStrPGM(PSTR("RDS TI V1.1 (software biphase) 06/2007 Init. \r\n\n"));
    RS_PrintfStrPGM(PSTR("RDS USB V1.2 (software biphase) 08/2007.\r\n\n"));
	#endif
    #ifndef RDS_SOFT_BIPHASE
    RS_PrintfStrPGM(PSTR("RDS TI V1.0 (hardware biphase) 06/2007 Init. \r\n\n"));
    #endif
    
    /* init USB driver */
    RS_USBIntefaceInit();
	/* autorisation IRQ */
    sei();            /* Enable interrupts => enable UART interrupts */

}



int main (void)
{
   /* initialise application */

   /* debug  Reset RAM  to check stack using */  
//    {
//     char * ptInit=(char *)0x19C;  /* first byte RAM 0x0060, first free RAM byte .bss   0x19c from file.lst */
//     
//     while (ptInit <(char *)(RAMEND-20))
//     {
//        *ptInit=0;
//        ptInit++;       
//     }
//    }
   
   InitAppli();
  
  
    /* home message before start RDS to avoid time problem with some too large string */
    RS_PrintfStrPGM(PSTR("RDS coder ATmega32  08/2007\r\n"));
    RS_PrintfStrPGM(PSTR("RS232 19200 Bds,8b data,2b stop, no parity\r\n"));
     RS_PrintfStrPGM(PSTR("R RDS current parameter (RAM) exple:\r\n"));
     RS_PrintfStrPGM(PSTR(" R PS <RDS NAME>\r\n"));
     RS_PrintfStrPGM(PSTR(" R PY 10      (PTY)\r\n"));
     RS_PrintfStrPGM(PSTR(" R RT <radiotext max 64c>\r\n"));
// not implemented in this version
//     RS_PrintfStrPGM(PSTR(" R AB 1       (text A/B)\r\n"));
     RS_PrintfStrPGM(PSTR(" R TP 1\r\n"));
     RS_PrintfStrPGM(PSTR(" R TA 0\r\n"));
     RS_PrintfStrPGM(PSTR(" R MS 1\r\n"));
     RS_PrintfStrPGM(PSTR(" R DI 1\r\n"));
     
     RS_PrintfStrPGM(PSTR("S Store EEPROM (used at board reset) same parameter as R commande adding following comands:\r\n"));
     RS_PrintfStrPGM(PSTR(" S NA 0       (AF number used : max 25)\r\n"));
     RS_PrintfStrPGM(PSTR(" S AF 0 877   (AF[0] in 100KHz)\r\n"));
     RS_PrintfStrPGM(PSTR(" S 2A 0       (2Agroup 0=desactivate / or 2A group every X 0A groups)\r\n"));
     RS_PrintfStrPGM(PSTR(" S ST 1       (1=stereo/0=mono)\r\n"));
     RS_PrintfStrPGM(PSTR(" S PI 61440   (in decimal base: 61440 = 0xF000)\r\n"));

     RS_PrintfStrPGM(PSTR("G (start/stop RDS generating (G 0 to stop)\r\n"));
     
     RS_PrintfStrPGM(PSTR("E (Erase EEPROM data and reset to default values)\r\n"));
// not available command, debug purpose only
//     RS_PrintfStrPGM(PSTR("D  RDS dump buffer (D 1)\r\n"));
     RS_PrintfStrPGM(PSTR("? Help menu \r\n"));
     
     {
       unsigned char TmpI;
 
 	/*dump EEPROM at start time to avoid timing problem when RDS streaming is running */
 	RS_PrintfStrPGM(PSTR("PICode: "));
     RS_PrintfUi(EEP_get_PICode(),0);
 	RS_PrintfStrPGM(PSTR("\r\n"));
 
 	RS_PrintfStrPGM(PSTR("PTY: "));
     RS_PrintfUi(EEP_get_PTY(),0);
 	RS_PrintfStrPGM(PSTR("\r\n"));
     
 	RS_PrintfStrPGM(PSTR("PSNAME: \""));
     for (TmpI=0;TmpI<RDS_SIZE_PSNAME;TmpI++)
     {
         RS_Printfc(EEP_get_PSName(TmpI));
     }
 	RS_PrintfStrPGM(PSTR("\"\r\n"));
     
 	RS_PrintfStrPGM(PSTR("TP: "));
     RS_PrintfUi(EEP_get_TP(),0);
 	RS_PrintfStrPGM(PSTR("\r\n"));
    
 	RS_PrintfStrPGM(PSTR("TA: "));
     RS_PrintfUi(EEP_get_TA(),0);
 	RS_PrintfStrPGM(PSTR("\r\n"));
    
  	RS_PrintfStrPGM(PSTR("DI: "));
     RS_PrintfUi(EEP_get_DI(),0);
 	RS_PrintfStrPGM(PSTR("\r\n"));
 
 	RS_PrintfStrPGM(PSTR("MS: "));
     RS_PrintfUi(EEP_get_MS(),0);
 	RS_PrintfStrPGM(PSTR("\r\n"));
   
  	RS_PrintfStrPGM(PSTR("NbAF: "));
     RS_PrintfUi(EEP_get_NbAF(),0);
 	RS_PrintfStrPGM(PSTR("\r\n"));
 	TmpI=0;
     while (TmpI!=EEP_get_NbAF())
     {
          RS_PrintfStrPGM(PSTR("AF["));
          RS_PrintfUi(TmpI,0);
 	     RS_PrintfStrPGM(PSTR("]="));
          RS_PrintfUi(EEP_get_AF(TmpI),0);
  	     RS_PrintfStrPGM(PSTR("  "));
          TmpI++;
      }
 	 RS_PrintfStrPGM(PSTR("\r\n"));
 	
      RS_PrintfStrPGM(PSTR("RTXT: \""));
      for (TmpI=0;TmpI<RDS_SIZE_RADIOTEXT;TmpI++)
      {
          RS_Printfc(EEP_get_Radiotext(TmpI));
      }
 	 RS_PrintfStrPGM(PSTR("\"\r\n"));
  	 
     if(EEP_get_StereoMode()==0)
 	 {
       	 RS_PrintfStrPGM(PSTR("mono\r\n"));
     }
     else
     {
 	     RS_PrintfStrPGM(PSTR("stereo\r\n"));    
     }
      RS_PrintfStrPGM(PSTR("2A: "));
      RS_PrintfUi(EEP_get_GroupRate((RDS_GROUP_TYPE_2A & RDS_GROUP_TYPE_SELECT),(RDS_GROUP_TYPE_2A>>RDS_GROUP_TYPE_SHIFT)),0);
 	  RS_PrintfStrPGM(PSTR("\r\n"));



     } /* end dump EEPROM dump */
    
    
 





   RS_PrintfStrPGM(PSTR("\r\n>"));
   RDSCLK_Start();
   //RS_PrintfStrPGM(PSTR("RDS stream out started\r\n"));

   /* forever loop */
   while (1)
   {
	    /* main_loop(); */
	    
        /*check if command available */
                /* processing new command */
        RS_ReceiveMgt();
        /*encode RDS data group if a frame buffer is free */
        RDSGEN_CheckFreeBuf(); 
        /* polling USB */
        usbPoll();
               
                
   }
return (0);
}
