//    Test Integration  RDS clock driver  functions of RDS software 
//
// =====================================================
//  Description
//    This file include initialisation and management functions
//       of RDS clock driver 
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
//=======================================
// Author : G.Laroche www.g.laroche.free.fr
//=======================================
// History:
// 22/05/07    V1.0
//       creation 
//       derivative from PC utilities in C++ using LPT1, files :
//             SendRDS.cpp   09/2000   author: g.laroche
//=======================================


#include "RDS_ClockDriver.h"
#include "RDS.h"
#include "RDS_RS232.h"  /* for debug functionnalities */
#include "RDS_GroupsGenerate.h"
#include "RDS_IOMgt.h"


/* include system */
#include <avr/io.h>
#include <stdlib.h>
#include <avr/pgmspace.h>   
#include <avr/interrupt.h>
#include <avr/signal.h>

/* local constant */

/* static variable  */
static  unsigned char  RDS_CurrentData;  /* current data to send */
static  unsigned char  RDS_CurrentMsk;   /* current mask to select next bit to send */ 
static  unsigned char  RDS_Out1;   /* output of first XOR gate */ 
static  unsigned char  RDS_OutD;   /* output of D gate */ 
static  unsigned char  RDS_OutData;   /* output of second XOR gate with  ~ */ 


/* For test version only */
// /* predefined Test data , 4 frame of 0A group for a complete PS NAME="TEST RDS" NOAF no TP*/
// // static unsigned char RDS_TI_Data[4*RDS_TI_SIZE_FRAME] = {
// // 0xf0,0x0,0xb2,0x1,0x5a,0x3c,0x7e,0xe,0xb,0x25,0x51,0x15,0xfb,
// // 0xf0,0x0,0xb2,0x1,0x5a,0x67,0xee,0xe,0xb,0x25,0x4d,0x51,0xe9,
// // 0xf0,0x0,0xb2,0x1,0x5a,0x8b,0x5e,0xe,0xb,0x24,0x81,0x4a,0xe1,
// // 0xf0,0x0,0xb2,0x1,0x5b,0xe5,0x1e,0xe,0xb,0x25,0x11,0x4d,0x60
// // 
// // 
// // };
// 


/* RDS clock driver functions */


/*  */
void RDSCLK_Init(void)
{

  
  /* initialise OCR1A value to generate 1187.5 Hz from 57K with a toggle change on each compare */
  OCR1AH = 0;
  OCR1AL = RDS_CLOCK_DIVIDE_57K;
  /* CTC mode 4 : clear timer on OCR1A match ,Toggle OC1A on each OCR1A match */
  TCCR1A |= (1 << COM1A0);
  TCCR1B |= (1 << WGM12);
  /* configure CNT1, clock source T1 on rising edge */
  TCCR1B |= ((1 << CS12) | (1 << CS11) | (1 << CS10));


 
}


/* stop rds clock process */
void RDSCLK_Stop(void)
{
   /* disable interrupt  */
  #ifdef RDS_SOFT_BIPHASE
  /* disable interrupt on Output compare 1A */
  TIMSK &= ~(1<<OCIE1A);
  #endif
  
  #ifndef RDS_SOFT_BIPHASE
    /* disable interrupt INT1  */
    GICR &= ~(1<< INT1);
  #endif 

}

/* start rds clock process   */
void RDSCLK_Start(void)
{ 
 

   RDS_Out1=0;
   RDS_OutD=0;
   RDS_OutData=0;
   
   /* initialise CurrentMsk to RDS_INIT_MASK_BIT */
   RDS_CurrentMsk = RDS_INIT_MASK_BIT;
   /* read next data by calling function RDSGEN_ReadNextData() and store it into RDS_CurrentData */
   RDS_CurrentData = RDSGEN_ReadNextData();
  
  #ifdef RDS_SOFT_BIPHASE
  /* enable interrupt on Output compare 1A */
  TIMSK |= (1<<OCIE1A);
  TIFR  |= (1<<OCF1A);
  #endif
  
  #ifndef RDS_SOFT_BIPHASE
    /* configure INT1 , interrupt on falling edge only for hard biphase */
    MCUCR |= (1<< ISC11);
    GICR |= (1<< INT1);
  #endif 

}



/* Interrupt handler for sending next rds data */
/* for USB driver compatibility , new interrupt must be enable as soon as possible */
/*INTERRUPT RDS_CLK_IT_SendData(void)*/
#ifdef RDS_SOFT_BIPHASE
SIGNAL(SIG_OUTPUT_COMPARE1A)
#endif
#ifndef RDS_SOFT_BIPHASE
SIGNAL(SIG_INTERRUPT1)
#endif
{

#ifdef RDS_SOFT_BIPHASE
    /* Software biphase encoding OUTPUT compare1A handler */
    /* if rising edge clock (output lev 1) */
    if ((RDS_OC1A_OUT_REG_IN & (1<< RDS_OC1A_OUT_BIT)) !=0)
    {

         /* set OutData = (OutD XOR 1) , -> = ~OutD value */
         /* OutD = Out1, D active on rise edge */
        if (RDS_Out1!=0)
        {
           /* set to 0 output RDS data */
           RDS_DATA_OUT_REG &= ~(1<<RDS_DATA_OUT_BIT);
        }
        else
        {
           /* set to 1 output RDS data */
           RDS_DATA_OUT_REG |= (1<<RDS_DATA_OUT_BIT);
        }
        /* update output D gate */
        RDS_OutD=RDS_Out1;
        /* update first XOR input */
        RDS_Out1 = RDS_OutD ^ RDS_OutData;
    }
    /* processing falling edge clock */
    else
    {
        /* set OutData = (OutD XOR 0) , -> = OutD */
        if (RDS_OutD!=0)
        {
           /* set to 1 output RDS data */
           RDS_DATA_OUT_REG |= (1<<RDS_DATA_OUT_BIT);
        }
        else
        {
           /* set to 0 output RDS data */
           RDS_DATA_OUT_REG &= ~(1<<RDS_DATA_OUT_BIT);
        }
        /* set new out data (input first XOR) */
        RDS_OutData =0;
        if ((RDS_CurrentData & RDS_CurrentMsk)!=0)
        {
           RDS_OutData =1;
        }
        /* set new output of first XOR */
        RDS_Out1= RDS_OutData ^ RDS_OutD; 
 
 #endif
 #ifndef RDS_SOFT_BIPHASE
 
    /*hardware biphase encoding , INT1 handler */
    /* acknowledge interrupt */
   GIFR |= (1<< INTF1);
   /* enable other interrupt */ 
   sei();


    /* IF ((RDS_CurrentData & RDS_CurrentMsk ) !=0 ) */
    /* THEN */
    if ((RDS_CurrentData & RDS_CurrentMsk) !=0)
    {
       /* set to 1 output RDS data */
       RDS_DATA_OUT_REG |= (1<<RDS_DATA_OUT_BIT);
    }
    /* ELSE */
    else
    {
       /* set to 0 output RDS data */
       RDS_DATA_OUT_REG &= ~(1<<RDS_DATA_OUT_BIT);
    /* ENDIF */
    }
 #endif       
        /* select next bit, and if necessary next char  */
    
        /* logical shift right RDS_CurrentMsk */
        RDS_CurrentMsk = (RDS_CurrentMsk >> 1);  
        /* IF RDS_CurrentMsk == 0 */
        /* THEN */
        if (RDS_CurrentMsk ==0)
        {
           /* initialise CurrentMsk to RDS_INIT_MASK_BIT */
           RDS_CurrentMsk = RDS_INIT_MASK_BIT;
           /* read next data by calling function RDSGEN_ReadNextData() and store it into RDS_CurrentData */
           RDS_CurrentData = RDSGEN_ReadNextData();
        /* ENDIF */
        }
#ifdef RDS_SOFT_BIPHASE       
    } /* End processing falling edge */
#endif       

}

/* For test and debug only, trace every no waiting interrupt */
// #ifdef RDS_TI_VERSION
// SIGNAL (SIG_INTERRUPT0)
// {
//    RS_DebugPrintfStrPGM(PSTR("SIG_INTERRUPT0 !\r\n"));
// }
// #ifdef RDS_SOFT_BIPHASE
//   SIGNAL (SIG_INTERRUPT1)
//   {
//      RS_DebugPrintfStrPGM(PSTR("SIG_INTERRUPT1 !\r\n"));
//   }
// #endif
// SIGNAL (SIG_INTERRUPT2)
// {
//    RS_DebugPrintfStrPGM(PSTR("SIG_INTERRUPT2 !\r\n"));
// }
// SIGNAL (SIG_OUTPUT_COMPARE2)
// {
//    RS_DebugPrintfStrPGM(PSTR("SIG_OUTPUT_COMPARE2 !\r\n"));
// }
// SIGNAL (SIG_OVERFLOW2)
// {
//    RS_DebugPrintfStrPGM(PSTR("SIG_OVERFLOW2 !\r\n"));
// }
// SIGNAL (SIG_INPUT_CAPTURE1)
// {
//    RS_DebugPrintfStrPGM(PSTR("SIG_INPUT_CAPTURE1 !\r\n"));
// }
// #ifndef RDS_SOFT_BIPHASE
//   SIGNAL (SIG_OUTPUT_COMPARE1A)
//   {
//      RS_DebugPrintfStrPGM(PSTR("SIG_OUTPUT_COMPARE1A !\r\n"));
//   }
// #endif
// SIGNAL (SIG_OUTPUT_COMPARE1B)
// {
//    RS_DebugPrintfStrPGM(PSTR("SIG_OUTPUT_COMPARE1B !\r\n"));
// }
// SIGNAL (SIG_OVERFLOW1)
// {
//    RS_DebugPrintfStrPGM(PSTR("SIG_OVERFLOW1 !\r\n"));
// }
// SIGNAL (SIG_OUTPUT_COMPARE0)
// {
//    RS_DebugPrintfStrPGM(PSTR("SIG_OUTPUT_COMPARE0 !\r\n"));
// }
// SIGNAL (SIG_OVERFLOW0)
// {
//    RS_DebugPrintfStrPGM(PSTR("SIG_OVERFLOW0 !\r\n"));
// }
// SIGNAL (SIG_SPI)
// {
//    RS_DebugPrintfStrPGM(PSTR("SIG_SPI !\r\n"));
// }
// // SIGNAL (SIG_UART_RECV)
// // {
// //    RS_DebugPrintfStrPGM(PSTR("SIG_UART_RECV !\r\n"));
// // }
// // SIGNAL (SIG_UART_DATA)
// // {
// //    RS_DebugPrintfStrPGM(PSTR("SIG_UART_DATA !\r\n"));
// // }
// SIGNAL (SIG_UART_TRANS)
// {
//    RS_DebugPrintfStrPGM(PSTR("SIG_UART_TRANS !\r\n"));
// }
// SIGNAL (SIG_ADC)
// {
//    RS_DebugPrintfStrPGM(PSTR("SIG_ADC !\r\n"));
// }
// SIGNAL (SIG_EEPROM_READY)
// {
//    RS_DebugPrintfStrPGM(PSTR("SIG_EEPROM_READY !\r\n"));
// }
// SIGNAL (SIG_COMPARATOR)
// {
//    RS_DebugPrintfStrPGM(PSTR("SIG_COMPARATOR !\r\n"));
// }
// SIGNAL (SIG_2WIRE_SERIAL)
// {
//    RS_DebugPrintfStrPGM(PSTR("SIG_2WIRE_SERIAL !\r\n"));
// }
// SIGNAL (SIG_SPM_READY)
// {
//    RS_DebugPrintfStrPGM(PSTR("SIG_SPM_READY !\r\n"));
// }
// 
// 
// #endif






